/* 
 * DNet eBusiness Suite
 * Copyright: Nan21 Electronics SRL. All rights reserved.
 * Use is subject to license terms.
 */

package net.nan21.dnet.module.sd.invoice.domain.entity;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.QueryHint;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import net.nan21.dnet.core.domain.eventhandler.DefaultEventHandler;
import net.nan21.dnet.module.md.tx.fin.domain.entity.TxAmount;
import net.nan21.dnet.module.sd.invoice.domain.entity.SalesInvoice;
import net.nan21.dnet.module.sd.order.domain.entity.SalesOrder;
import org.eclipse.persistence.annotations.CascadeOnDelete;
import org.eclipse.persistence.annotations.Customizer;
import org.eclipse.persistence.config.HintValues;
import org.eclipse.persistence.config.QueryHints;
import org.eclipse.persistence.descriptors.DescriptorEvent;

@NamedQueries({
	@NamedQuery(
		name=SalesTxAmount.NQ_FIND_BY_ID,
		query="SELECT e FROM SalesTxAmount e WHERE e.clientId = :pClientId and e.id = :pId ",
		hints=@QueryHint(name=QueryHints.BIND_PARAMETERS, value=HintValues.TRUE)
	)
	,@NamedQuery(
		name=SalesTxAmount.NQ_FIND_BY_IDS,
		query="SELECT e FROM SalesTxAmount e WHERE e.clientId = :pClientId and e.id in :pIds",
		hints=@QueryHint(name=QueryHints.BIND_PARAMETERS, value=HintValues.TRUE)
	)
})
@Entity
@CascadeOnDelete
@Table(
	name=SalesTxAmount.TABLE_NAME
)
@Customizer(DefaultEventHandler.class)
public class SalesTxAmount extends TxAmount  {
	
	public static final String TABLE_NAME = "SD_TX_AMNT";
	public static final String SEQUENCE_NAME = "SD_TX_AMNT_SEQ";
	
	private static final long serialVersionUID = -8865917134914502125L;
	
	/**
	 * Named query find by ID.
	 */ 
	public static final String NQ_FIND_BY_ID = "SalesTxAmount.findById";
	
	/**
	 * Named query find by IDs.
	 */     
	public static final String NQ_FIND_BY_IDS = "SalesTxAmount.findByIds";
	
	/**
			 * System generated unique identifier.
			 */
	@Column(name="ID", nullable=false)
	@NotNull
	@Id
	@GeneratedValue(generator=SEQUENCE_NAME)
	private Long id;
	
	@ManyToOne(fetch=FetchType.LAZY, targetEntity=SalesOrder.class)
	@JoinColumn(name="ORDER_ID", referencedColumnName="ID")
	private SalesOrder order;
	
	@ManyToOne(fetch=FetchType.LAZY, targetEntity=SalesInvoice.class)
	@JoinColumn(name="INVOICE_ID", referencedColumnName="ID")
	private SalesInvoice invoice;
	
	public Long getId() {
		return this.id;
	}
	
	public void setId(Long id) {
		this.id = id;
	}
	
	public SalesOrder getOrder() {
		return this.order;
	}
	
	public void setOrder(SalesOrder order) {
		if (order != null ) {
			this.__validate_client_context__(order.getClientId());
		}
		this.order = order;
	}
	
	public SalesInvoice getInvoice() {
		return this.invoice;
	}
	
	public void setInvoice(SalesInvoice invoice) {
		if (invoice != null ) {
			this.__validate_client_context__(invoice.getClientId());
		}
		this.invoice = invoice;
	}
	
	public void aboutToInsert(DescriptorEvent event) {
		super.aboutToInsert(event);
	
	}
}
